<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\DependencyInjection\Compiler;

use Sylius\Component\Core\Model\ChannelPricingInterface;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Exception\InvalidArgumentException;

/**
 * TLDR : Due to the fact that we make ChannelPricing an inheritable entity, we need to resolve the target entity for ChannelPricingInterface
 *
 * See Sylius\Bundle\ResourceBundle\DependencyInjection\Compiler\DoctrineTargetEntitiesResolverPass
 * See Sylius\Bundle\ResourceBundle\DependencyInjection\Compiler\Helper\TargetEntitiesResolver
 *
 * In the TargetEntitiesResolver, Sylius uses the configure sylius_resources to resolve interfaces and model mapping
 * This allows to declare association mapping based on Interfaces (eg : channelPricing->productVariant with Sylius\Component\Core\Model\ProductVariantInterface as the target entity)
 * The DoctrineTargetEntitiesResolverPass makes it so that Doctrine nows that ProductVariantInterface refers to the actuel model implementation declared in sylius_resources config (whether you declare Sylius\Component\Core\Model\ProductVariant or App\Entity\ProductVariant or anything)
 *
 * To avoid having funny things declared to Doctrine, Sylius make it so that when an interface is declared in several resources, the TargetEntitiesResolver ignore the interface
 * This prevents a reusable interface to be mapped on several models (eg : CodeAwareInterface is ignored because it is declared on multiple model implementations)
 *
 * In our case, we extend ChannelPricing (which is a Sylius\Component\Core\Model\ChannelPricingInterface) with our children entities CustomerChannelPricing, OrganizationChannelPricing and CustomerGroupChannelPricing
 * The TargetEntitiesResolver will ignore ChannelPricingInterface because it is declared on multiple model implementations
 * This will raise exceptions as ChannelPricingInterface is used in association mapping
 *
 * This CompilerPass fixes this by explicitly resolving the target entity for ChannelPricingInterface
 */
class DoctrineChannelPricingTargetEntityResolverPass implements CompilerPassInterface
{
    public function process(ContainerBuilder $container)
    {
        try {
            /** @var array $resources */
            $resources = $container->getParameter('sylius.resources');
            $resolveTargetEntityListener = $container->findDefinition('doctrine.orm.listeners.resolve_target_entity');
        } catch (InvalidArgumentException $exception) {
            return;
        }

        $resolveTargetEntityListener->addMethodCall('addResolveTargetEntity', [ChannelPricingInterface::class, $resources['sylius.channel_pricing']['classes']['model'], []]);
    }
}
